@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -32,7 +32,7 @@
 :- module(win_menu,
 	  [ init_win_menus/0
 	  ]).
-:- style_check(+dollar).
+:- set_prolog_flag(generate_debug_info, false).
 :- op(200, fy, @).
 :- op(990, xfx, :=).
 
@@ -58,8 +58,8 @@ menu('&File',
      ]).
 menu('&Settings',
      [ --,
-       '&User init file ...' = prolog_edit_preferences(prolog),
-       '&Stack sizes ...'    = prolog_edit_preferences(stack_sizes)
+       '&User init file ...'  = prolog_edit_preferences(prolog),
+       '&GUI preferences ...' = prolog_edit_preferences(xpce)
      ],
      []).
 menu('&Debug',
@@ -77,8 +77,8 @@ menu('&Debug',
      [ before_menu(-)
      ]).
 menu('&Help',
-     [ '&About ...' 				= about,
-       '&Help ...' 				= help,
+     [ '&About ...'				= about,
+       '&Help ...'				= help,
        'SWI-Prolog &Manual (on www) ...'	= www_open(swipl_man),
        --,
        'SWI-Prolog &WWW home (on www) ...'	= www_open(swipl),
@@ -87,9 +87,9 @@ menu('&Help',
        'SWI-Prolog Mailing &List (on www) ...'	= www_open(swipl_mail),
        'SWI-Prolog &Download (on www) ...'	= www_open(swipl_download),
        --,
-       '&XPCE (GUI) Manual ...' 		= manpce,
+       '&XPCE (GUI) Manual ...'			= manpce,
        --,
-       'Submit &Bug report (on www) ...'  	= www_open(swipl_bugs)
+       'Submit &Bug report (on www) ...'	= www_open(swipl_bugs)
      ],
      [ before_menu(-)
      ]).
@@ -181,7 +181,7 @@ gather_arg(file(Mode, Title), File) :-
 	Filter =.. [chain|AllTuples],
 	current_prolog_flag(hwnd, HWND),
 	working_directory(CWD, CWD),
-	call(get(@display, win_file_name, 	% avoid autoloading
+	call(get(@display, win_file_name,	% avoid autoloading
 		 Mode, Filter, Title,
 		 directory := CWD,
 		 owner := HWND,
