@@ -40,7 +40,7 @@
 :- use_module(library(http/http_parameters)).
 :- use_module(library(http/html_write)).
 :- use_module(library(http/mimetype)).
-:- use_module(library(http/dcg_basics)).
+:- use_module(library(dcg/basics)).
 :- use_module(library(http/http_dispatch)).
 :- use_module(library(http/http_hook)).
 :- use_module(library(http/http_path)).
@@ -267,10 +267,21 @@ pldoc_index(_Request) :-
 			title('SWI-Prolog documentation'),
 			[ \doc_links('', []),
 			   h1('SWI-Prolog documentation'),
-			  \man_overview([])
+			  \man_overview([]),
+			  \paperback
 			]).
 
 
+paperback -->
+	{ expand_url_path(swipl_book(.), HREF)
+	},
+	html([ h2('The manual as a book'),
+	       p([ 'A paperback version of the manual is ',
+		   a(href(HREF), 'available'), '.'
+		 ])
+	     ]).
+
+
 %%	pldoc_file(+Request)
 %
 %	Hander for /file?file=File, providing documentation for File.
@@ -476,6 +487,10 @@ documentation(Path, Request) :-
 	->  true
 	;   throw(http_reply(forbidden(File)))
 	),
+        (   exists_file(File)
+        ->  true
+        ;   throw(http_reply(not_found(File)))
+        ),
 	(   Reload == true,
 	    source_file(File)
 	->  load_files(File, [if(changed), imports([])])
