@@ -42,6 +42,7 @@
 	  ]).
 :- use_module(library(lists)).
 :- use_module(library(debug)).
+:- use_module(library(error)).
 :- use_module(library(memfile)).
 :- use_module(library(pairs)).
 :- use_module(library(option)).
@@ -738,7 +739,10 @@ nv_pair(Option) -->
 	[ w(Name), =,'"'], tokens(Tokens), ['"'], !,
 	{ untag(Tokens, Atoms),
 	  atomic_list_concat(Atoms, Value0),
-	  catch(atom_number(Value0, Value), _, Value=Value0),
+	  (   atom_number(Value0, Value)
+	  ->  true
+	  ;   Value = Value0
+	  ),
 	  Option =.. [Name,Value]
 	}.
 
@@ -923,6 +927,7 @@ functor_name(String) :-
 	char_type(Char, lower).
 
 url_protocol(http).
+url_protocol(https).
 url_protocol(ftp).
 url_protocol(mailto).
 
