@@ -1262,17 +1262,18 @@ rdf_storage_encoding('', plain).
 rdf_load_stream(xml, Stream, Options) :- !,
 	graph(Options, Graph),
 	rdf_transaction(process_rdf(Stream, assert_triples, Options),
-			parse(Graph)).
+			parse(Graph)),
+	rdf_graph_clear_modified_(Graph).
 rdf_load_stream(xhtml, Stream, M:Options) :- !,
 	graph(Options, Graph),
 	rdf_transaction(process_rdf(Stream, assert_triples,
 				    M:[embedded(true)|Options]),
-			parse(Graph)).
+			parse(Graph)),
+	rdf_graph_clear_modified_(Graph).
 rdf_load_stream(triples, Stream, Options) :- !,
 	graph(Options, Graph),
 	rdf_load_db_(Stream, Graph, _Graphs).
 
-
 %%	report_loaded(+Action, +Source, +DB, +Triples, +StartCPU, +Options)
 
 report_loaded(none, _, _, _, _, _) :- !.
@@ -1402,7 +1403,7 @@ rdf_graph_property_(triples(Count), Graph) :-
 %	    Set the modified state of Graph to false.
 
 rdf_set_graph(Graph, modified(Modified)) :-
-	must_be(Modified, oneof([false])),
+	must_be(oneof([false]), Modified),
 	rdf_graph_clear_modified_(Graph).
 
 
@@ -2158,7 +2159,13 @@ save_body_literal(lang(Lang, Value),
 save_body_literal(type(Type, DOM),
 		  NameText, _BaseURI, Out, Indent, Options) :-
 	rdf_equal(Type, rdf:'XMLLiteral'), !,
-	save_xml_literal(DOM, NameText, Out, Indent, Options).
+	(   atom(DOM)
+	->  format(Out, '~N~*|<', [Indent]),
+	    rdf_write_id(Out, NameText),
+	    format(Out, ' rdf:parseType="Literal">~w</', [DOM]),
+	    rdf_write_id(Out, NameText), write(Out, '>')
+	;   save_xml_literal(DOM, NameText, Out, Indent, Options)
+	).
 save_body_literal(type(Type, Value),
 		  NameText, BaseURI, Out, Indent, _) :- !,
 	format(Out, '~N~*|<', [Indent]),
