@@ -1,11 +1,10 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 2002-2012, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -38,14 +37,15 @@
 	    rdf_journal_file/2,		% ?DB, ?JournalFile
 	    rdf_db_to_file/2		% ?DB, ?FileBase
 	  ]).
-:- use_module(library('semweb/rdf_db')).
+:- use_module(library(semweb/rdf_db)).
+:- use_module(library(filesex)).
 :- use_module(library(lists)).
-:- use_module(library(url)).
+:- use_module(library(uri)).
 :- use_module(library(debug)).
 :- use_module(library(error)).
 :- use_module(library(thread)).
 :- use_module(library(pairs)).
-:- use_module(library(time)).
+:- use_module(library(apply)).
 
 
 /** <module> RDF persistency plugin
@@ -68,8 +68,14 @@ is restored by loading  the  snapshot   and  replaying  the journal. The
 predicate rdf_flush_journals/1 can be used to create fresh snapshots and
 delete the journals.
 
-@tbd if there is a complete `.new'   snapshot  and no journal, we should
-move the .new to the plain snapshot name as a means of recovery.
+@tbd If there is a complete `.new'   snapshot  and no journal, we should
+     move the .new to the plain snapshot name as a means of recovery.
+
+@tbd Backup of each graph using one or two files is very costly if there
+     are many graphs.  Although the currently used subdirectories avoid
+     hitting OS limits early, this is still not ideal. Probably we
+     should collect (small, older?) files and combine them into a single
+     quick load file.  We could call this (similar to GIT) a `pack'.
 
 @see	rdf_edit.pl
 */
@@ -79,14 +85,12 @@ move the .new to the plain snapshot name as a means of recovery.
 	rdf_lock/2,
 	rdf_option/1,
 	source_journal_fd/2,
-	db_file_base/2,
 	file_base_db/2.
 :- dynamic
 	rdf_directory/1,		% Absolute path
 	rdf_lock/2,			% Dir, Lock
 	rdf_option/1,			% Defined options
 	source_journal_fd/2,		% DB, JournalFD
-	db_file_base/2,			% DB, FileBase
 	file_base_db/2.			% FileBase, DB
 
 :- meta_predicate
@@ -121,6 +125,10 @@ move the .new to the plain snapshot name as a means of recovery.
 %		Maximum number of journals kept open.  If not provided,
 %		the default is 10.  See limit_fd_pool/0.
 %
+%		* directory_levels(+Count)
+%		Number of levels of intermediate directories for storing
+%		the graph files.  Default is 2.
+%
 %		* silent(+BoolOrBrief)
 %		If =true= (default =false=), do not print informational
 %		messages.  Finally, if =brief= it will show minimal
@@ -173,6 +181,7 @@ assert_options([H|T]) :-
 
 option_type(concurrency(X),		must_be(positive_integer, X)).
 option_type(max_open_journals(X),	must_be(positive_integer, X)).
+option_type(directory_levels(X),	must_be(positive_integer, X)).
 option_type(silent(X),	       must_be(oneof([true,false,brief]), X)).
 option_type(log_nested_transactions(X),	must_be(boolean, X)).
 
@@ -185,8 +194,10 @@ option_type(log_nested_transactions(X),	must_be(boolean, X)).
 
 no_agc(Goal) :-
 	current_prolog_flag(agc_margin, Old),
-	set_prolog_flag(agc_margin, 0),
-	call_cleanup(Goal, set_prolog_flag(agc_margin, Old)).
+	setup_call_cleanup(
+	    set_prolog_flag(agc_margin, 0),
+	    Goal,
+	    set_prolog_flag(agc_margin, Old)).
 
 
 %%	rdf_detach_db is det.
@@ -204,7 +215,6 @@ rdf_detach_db :-
 	    save_prefixes(Dir),
 	    retractall(rdf_option(_)),
 	    retractall(source_journal_fd(_,_)),
-	    retractall(db_file_base(_,_)),
 	    unlock_db(Dir)
 	;   true
 	).
@@ -256,11 +266,10 @@ rdf_flush_journal(DB, Options) :-
 
 load_db :-
 	rdf_directory(Dir),
-	load_prefixes(Dir),
-	working_directory(Old, Dir),
 	get_time(Wall0),
 	statistics(cputime, T0),
-	call_cleanup(find_dbs(DBs), working_directory(_, Old)),
+	load_prefixes(Dir),
+	find_dbs(Dir, DBs),
 	length(DBs, DBCount),
 	verbosity(DBCount, Silent),
 	make_goals(DBs, Silent, 1, DBCount, Goals),
@@ -282,10 +291,7 @@ make_goals([DB|T0], Silent, I,  Total,
 
 verbosity(_DBCount, Silent) :-
 	rdf_option(silent(Silent)), !.
-verbosity(DBCount, Silent) :-
-	DBCount > 25, !,
-	Silent = brief.
-verbosity(_DBCount, false).
+verbosity(_DBCount, brief).
 
 
 %%	concurrency(-Jobs)
@@ -300,50 +306,93 @@ concurrency(Jobs) :-
 concurrency(1).
 
 
-%%	find_dbs(-DBs:list(atom)) is det.
+%%	find_dbs(+Dir, -DBs:list(atom), -Depth) is det.
 %
 %	DBs is a  list  of  database   (named  graph)  names,  sorted in
 %	increasing file-size. Small files  are   loaded  first  as these
 %	typically contain the schemas and we   want  to avoid re-hashing
 %	large databases due to added rdfs:subPropertyOf triples.
+%
+%	@param Depth is the depth at which all files have been found.
 
-find_dbs(DBs) :-
-	expand_file_name(*, Files),
-	phrase(scan_db_files(Files), Scanned),
+find_dbs(Dir, DBs) :-
+	find_dbs(Dir, DBs, Depth),
+	(   var(Depth)			% empty DB
+	->  true
+	;   length(DBs, Count),
+	    LevelsRequired is floor(log(Count)/log(256)),
+	    Depth < LevelsRequired
+	->  print_message(informational, rdf(reindex(Count, LevelsRequired))),
+	    reindex_db(Dir, LevelsRequired),
+	    retractall(rdf_option(directory_levels(_))),
+	    assertz(rdf_option(directory_levels(LevelsRequired)))
+	;   retractall(rdf_option(directory_levels(_))),
+	    assertz(rdf_option(directory_levels(Depth)))
+	).
+
+find_dbs(Dir, DBs, Depth) :-
+	directory_files(Dir, Files),
+	phrase(scan_db_files(Files, Dir, '.', 0), Scanned),
 	sort(Scanned, ByDB),
 	join_snapshot_and_journals(ByDB, BySize),
 	keysort(BySize, SortedBySize),
-	pairs_values(SortedBySize, DBs).
+	pairs_values(SortedBySize, DBs),
+	(   maplist(depth(Depth), Scanned)
+	->  true
+	;   length(DBs, Count),
+	    Levels is floor(log(Count)/log(256)),
+	    print_message(warning, rdf(fix_index(Levels))),
+	    reindex_db(Dir, Levels)
+	).
 
+depth(Depth, DB) :-
+	arg(4, DB, Depth).
 
-%%	scan_db_files(+Files)// is det.
+%%	scan_db_files(+Files, +Dir, +Prefix, +Depth)// is det.
 %
 %	Produces a list of db(DB,  Size,   File)  for all recognised RDF
-%	database files.
+%	database files.  File is relative to the database directory Dir.
 
-scan_db_files([]) -->
+scan_db_files([], _, _, _) -->
 	[].
-scan_db_files([File|T]) -->
+scan_db_files([Nofollow|T], Dir, Prefix, Depth) -->
+	{ nofollow(Nofollow) }, !,
+	scan_db_files(T, Dir, Prefix, Depth).
+scan_db_files([File|T], Dir, Prefix, Depth) -->
 	{ file_name_extension(Base, Ext, File),
 	  db_extension(Ext), !,
 	  rdf_db_to_file(DB, Base),
-	  size_file(File, Size)
+	  directory_file_path(Prefix, File, DBFile),
+	  directory_file_path(Dir, DBFile, AbsFile),
+	  size_file(AbsFile, Size)
 	},
-	[ db(DB, Size, File) ],
-	scan_db_files(T).
-scan_db_files([_|T]) -->
-	scan_db_files(T).
+	[ db(DB, Size, DBFile, Depth) ],
+	scan_db_files(T, Dir, Prefix, Depth).
+scan_db_files([D|T], Dir, Prefix, Depth) -->
+	{ directory_file_path(Prefix, D, SubD),
+	  directory_file_path(Dir, SubD, AbsD),
+	  exists_directory(AbsD),
+	  \+ read_link(AbsD, _, _), !,	% Do not follow links
+	  directory_files(AbsD, SubFiles),
+	  SubDepth is Depth + 1
+	},
+	scan_db_files(SubFiles, Dir, SubD, SubDepth),
+	scan_db_files(T, Dir, Prefix, Depth).
+scan_db_files([_|T], Dir, Prefix, Depth) -->
+	scan_db_files(T, Dir, Prefix, Depth).
 
+nofollow(.).
+nofollow(..).
 
 db_extension(trp).
 db_extension(jrn).
 
 join_snapshot_and_journals([], []).
-join_snapshot_and_journals([db(DB,S0,_)|T0], [S-DB|T]) :- !,
+join_snapshot_and_journals([db(DB,S0,_,_)|T0], [S-DB|T]) :- !,
 	same_db(DB, T0, T1, S0, S),
 	join_snapshot_and_journals(T1, T).
 
-same_db(DB, [db(DB,S1,_)|T0], T, S0, S) :- !,
+same_db(DB, [db(DB,S1,_,_)|T0], T, S0, S) :- !,
 	S2 is S0+S1,
 	same_db(DB, T0, T, S2, S).
 same_db(_, L, L, S, S).
@@ -832,7 +881,12 @@ create_db(DB) :-
 	debug(rdf_persistency, 'Saving DB ~w', [DB]),
 	db_abs_files(DB, Snapshot, Journal),
 	atom_concat(Snapshot, '.new', NewSnapshot),
-	(   catch(rdf_save_db(NewSnapshot, DB), _, fail)
+	(   catch(( create_directory_levels(Snapshot),
+		    rdf_save_db(NewSnapshot, DB)
+		  ), Error,
+		  ( print_message(warning, Error),
+		    fail
+		  ))
 	->  (   exists_file(Journal)
 	    ->  delete_file(Journal)
 	    ;   true
@@ -921,18 +975,52 @@ unlock_db(Out, File) :-
 lockfile(Dir, LockFile) :-
 	atomic_list_concat([Dir, /, lock], LockFile).
 
+directory_levels(Levels) :-
+	rdf_option(directory_levels(Levels)), !.
+directory_levels(2).
+
 db_file(Base, File) :-
 	rdf_directory(Dir),
-	atomic_list_concat([Dir, /, Base], File).
+	directory_levels(Levels),
+	db_file(Dir, Base, Levels, File).
+
+db_file(Dir, Base, Levels, File) :-
+	dir_levels(Base, Levels, Segments, [Base]),
+	atomic_list_concat([Dir|Segments], /, File).
 
 open_db(Base, Mode, Stream, Options) :-
 	db_file(Base, File),
+	create_directory_levels(File),
 	open(File, Mode, Stream, [encoding(utf8)|Options]).
 
+create_directory_levels(_File) :-
+	rdf_option(directory_levels(0)), !.
+create_directory_levels(File) :-
+	file_directory_name(File, Dir),
+	make_directory_path(Dir).
+
 exists_db(Base) :-
 	db_file(Base, File),
 	exists_file(File).
 
+%%	dir_levels(+File, +Levels, ?Segments, ?Tail) is det.
+%
+%	Create a list of intermediate directory names for File.  Each
+%	directory consists of two hexadecimal digits.
+
+dir_levels(_, 0, Segments, Segments) :- !.
+dir_levels(File, Levels, Segments, Tail) :-
+	rdf_atom_md5(File, 1, Hash),
+	create_dir_levels(Levels, 0, Hash, Segments, Tail).
+
+create_dir_levels(0, _, _, Segments, Segments) :- !.
+create_dir_levels(N, S, Hash, [S1|Segments0], Tail) :-
+	sub_atom(Hash, S, 2, _, S1),
+	S2 is S+2,
+	N2 is N-1,
+	create_dir_levels(N2, S2, Hash, Segments0, Tail).
+
+
 %%	db_files(+DB, -Snapshot, -Journal).
 %%	db_files(-DB, +Snapshot, -Journal).
 %%	db_files(-DB, -Snapshot, +Journal).
@@ -989,20 +1077,11 @@ rdf_journal_file(DB, Journal) :-
 %	reasons. Speed, but much more important   is that the mapping of
 %	raw --> encoded provided by  www_form_encode/2 is not guaranteed
 %	to be unique by the W3C standards.
-%
-%	@tbd	We keep two predicates for exploiting Prolog indexing.
-%		Once multi-argument indexed is hashed we should clean
-%		this up.
 
 rdf_db_to_file(DB, File) :-
-	nonvar(File),
 	file_base_db(File, DB), !.
-rdf_db_to_file(DB, File) :-
-	nonvar(DB),
-	db_file_base(DB, File), !.
 rdf_db_to_file(DB, File) :-
 	url_to_filename(DB, File),
-	assert(db_file_base(DB, File)),
 	assert(file_base_db(File, DB)).
 
 %%	url_to_filename(+URL, -FileName) is det.
@@ -1022,7 +1101,7 @@ url_to_filename(URL, FileName) :-
 	phrase(url_encode(EncCodes), Codes),
 	atom_codes(FileName, EncCodes).
 url_to_filename(URL, FileName) :-
-	www_form_encode(URL, FileName).
+	uri_encoded(path, URL, FileName).
 
 url_encode([0'+|T]) -->
 	" ", !,
@@ -1062,6 +1141,70 @@ alphanum(C) -->
 no_enc_extra(0'_) --> "_".
 
 
+		 /*******************************
+		 *	       REINDEX		*
+		 *******************************/
+
+%%	reindex_db(+Dir)
+%
+%	Reindex the database by creating intermediate directories.
+
+reindex_db(Dir) :-
+	directory_levels(Levels),
+	reindex_db(Dir, Levels).
+
+reindex_db(Dir, Levels) :-
+	directory_files(Dir, Files),
+	reindex_files(Files, Dir, '.', 0, Levels),
+	remove_empty_directories(Files, Dir).
+
+reindex_files([], _, _, _, _).
+reindex_files([Nofollow|Files], Dir, Prefix, CLevel, Levels) :-
+	nofollow(Nofollow), !,
+	reindex_files(Files, Dir, Prefix, CLevel, Levels).
+reindex_files([File|Files], Dir, Prefix, CLevel, Levels) :-
+	CLevel \== Levels,
+	file_name_extension(_Base, Ext, File),
+	db_extension(Ext), !,
+	directory_file_path(Prefix, File, DBFile),
+	directory_file_path(Dir, DBFile, OldPath),
+	db_file(Dir, File, Levels, NewPath),
+	debug(rdf_persistency, 'Rename ~q --> ~q', [OldPath, NewPath]),
+	file_directory_name(NewPath, NewDir),
+	make_directory_path(NewDir),
+	rename_file(OldPath, NewPath),
+	reindex_files(Files, Dir, Prefix, CLevel, Levels).
+reindex_files([D|Files], Dir, Prefix, CLevel, Levels) :-
+	directory_file_path(Prefix, D, SubD),
+	directory_file_path(Dir, SubD, AbsD),
+	exists_directory(AbsD),
+	\+ read_link(AbsD, _, _), !,	% Do not follow links
+	directory_files(AbsD, SubFiles),
+	CLevel2 is CLevel + 1,
+	reindex_files(SubFiles, Dir, SubD, CLevel2, Levels),
+	reindex_files(Files, Dir, Prefix, CLevel, Levels).
+reindex_files([_|Files], Dir, Prefix, CLevel, Levels) :-
+	reindex_files(Files, Dir, Prefix, CLevel, Levels).
+
+
+remove_empty_directories([], _).
+remove_empty_directories([File|Files], Dir) :-
+	\+ nofollow(File),
+	directory_file_path(Dir, File, Path),
+	exists_directory(Path),
+	\+ read_link(Path, _, _), !,
+	directory_files(Path, Content),
+	exclude(nofollow, Content, RealContent),
+	(   RealContent == []
+	->  debug(rdf_persistency, 'Remove empty dir ~q', [Path]),
+	    delete_directory(Path)
+	;   remove_empty_directories(RealContent, Path)
+	),
+	remove_empty_directories(Files, Dir).
+remove_empty_directories([_|Files], Dir) :-
+	remove_empty_directories(Files, Dir).
+
+
 		 /*******************************
 		 *	      PREFIXES		*
 		 *******************************/
@@ -1162,6 +1305,10 @@ message(restore(_, done(_, Time, Count, _Nth, _Total))) -->
 	[ at_same_line, '~D triples in ~2f sec.'-[Count, Time] ].
 message(update_failed(S,P,O,Action)) -->
 	[ 'Failed to update <~p ~p ~p> with ~p'-[S,P,O,Action] ].
+message(reindex(Count, Depth)) -->
+	[ 'Restructuring database with ~d levels (~D graphs)'-[Depth, Count] ].
+message(reindex(Depth)) -->
+	[ 'Fixing database directory structure (~d levels)'-[Depth] ].
 
 silent_message(_Action) --> [].
 
