@@ -76,6 +76,8 @@ typedef struct _winassoc
 } winassoc, *WinAssoc;
 
 static WinAssoc wintable[WINASSOC_TABLESIZE];
+static int		lock_initialized;
+static CRITICAL_SECTION lock;
 
 static int
 handleKey(HWND handle)
@@ -95,20 +97,28 @@ assocObjectToHWND(HWND hwnd, Any obj)
   WinAssoc *p = &wintable[key];
   WinAssoc  a = *p;
 
+  if ( !lock_initialized )		/* we are serialized by the XPCE */
+  { lock_initialized = TRUE;		/* lock, so this must be safe */
+    InitializeCriticalSection(&lock);
+  }
+
   if ( isNil(obj) )			/* delete from table */
-  { for( ; a ; p = &a->next, a = a->next )
+  { EnterCriticalSection(&lock);
+    for( ; a ; p = &a->next, a = a->next )
     { if ( a->hwnd == hwnd )
       { *p = a->next;
         unalloc(sizeof(winassoc), a);
 	return;
       }
     }
+    LeaveCriticalSection(&lock);
 					/* not in the table!? */
   } else
   { WinAssoc n = alloc(sizeof(winassoc));
-    n->next   = *p;
+
     n->hwnd   = hwnd;
     n->object = obj;
+    n->next   = *p;
     *p = n;
   }
 
@@ -139,33 +149,59 @@ destroyThreadWindows(Class class)
     *destroy* such objects.  We could also have choosen for `uncreate',
     leaving the object around but destroying the Windows.
 
-    This routine is called from DllMain() in mswin.c
+    This routine is called from pceMTdetach() in mswin.c
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 void
 destroyThreadWindows(Class class)
-{ Chain ch = newObject(ClassChain, EAV);
+{ int found = FALSE;
   int i;
   DWORD tid = GetCurrentThreadId();
   Any obj;
 
+  if ( !lock_initialized )		/* there are no windows */
+    return;
+
+  EnterCriticalSection(&lock);
   for(i=0; i<WINASSOC_TABLESIZE; i++)
   { WinAssoc a;
 
     for(a = wintable[i]; a; a = a->next)
-    { if ( instanceOfObject(a->object, class) &&
-	   tid == GetWindowThreadProcessId(a->hwnd, NULL) )
-	appendChain(ch, a->object);
+    { if ( tid == GetWindowThreadProcessId(a->hwnd, NULL) )
+      { found = TRUE;
+	goto out;
+      }
     }
   }
+out:
+  LeaveCriticalSection(&lock);
 
-  for_chain(ch, obj,
-	    { DEBUG(NAME_thread, Cprintf("Destroying %s owned by 0x%x\n",
-					 pp(obj), tid));
-	      sendv(obj, NAME_destroy, 0, NULL);
-	    });
+  if ( found )
+  { Chain ch;
 
-  freeObject(ch);
+    DEBUG(NAME_thread,
+	  Cprintf("Thread has associated windows; destroying ...\n"));
+    pceMTLock(LOCK_PCE);
+    ch = newObject(ClassChain, EAV);
+
+    for(i=0; i<WINASSOC_TABLESIZE; i++)
+    { WinAssoc a;
+
+      for(a = wintable[i]; a; a = a->next)
+      { if ( instanceOfObject(a->object, class) &&
+	     tid == GetWindowThreadProcessId(a->hwnd, NULL) )
+	  appendChain(ch, a->object);
+      }
+    }
+
+    for_chain(ch, obj,
+	      { DEBUG(NAME_thread, Cprintf("Destroying %s owned by 0x%x\n",
+					   pp(obj), tid));
+		sendv(obj, NAME_destroy, 0, NULL);
+	      });
+    freeObject(ch);
+    pceMTUnlock(LOCK_PCE);
+  }
 }
 
 
@@ -263,13 +299,54 @@ ws_emulate_three_buttons(int time)
 }
 
 
+/* Adapted from code by Jeroen van der Zijp for FX.  Bottom line is that
+   on some keyboards (certainly French and German), AltGR sends CONTROL
+   and ALT.
+*/
+
+static int
+ws_has_alt_gr(void)
+{ static int has_altgr = FALSE;
+  static HKL hklOld = NULL;
+  HKL hkl=GetKeyboardLayout(0);
+
+  if ( hklOld != hkl )
+  { unsigned int ch;
+
+    hklOld = hkl;
+    has_altgr = FALSE;
+
+    for(ch=0x20; ch<=0xff ; ++ch)
+    { // <MSDN>
+      // For keyboard layouts that use the right-hand ALT key as a shift key
+      // (for example, the French keyboard layout), the shift state is
+      // represented by the value 6, because the right-hand ALT key is
+      // converted internally into CTRL+ALT.
+      // </MSDN>
+      if ( HIBYTE(VkKeyScanEx(ch,hkl)) == 6 )
+      { has_altgr = TRUE;
+	break;
+      }
+    }
+
+    DEBUG(NAME_key, Cprintf("has_altgr = %d\n", has_altgr));
+  }
+
+  return has_altgr;
+}
+
+
 static int
 IsDownKey(code)
 { short mask = GetKeyState(code);
+  int down = (mask & 0x8000) != 0;
 
   DEBUG(NAME_key, Cprintf("IsDownKey(%d): mask = 0x%x\n", code, mask));
+  if ( code == VK_CONTROL && ws_has_alt_gr() &&
+       (GetKeyState(VK_MENU) & 0x8000) )
+    return FALSE;
 
-  return mask & 0x8000;
+  return down;
 }
 
 
@@ -287,7 +364,9 @@ messageToKeyId(UINT message, UINT wParam, LONG lParam, unsigned long *bmask)
 
   switch(message)
   { case WM_KEYDOWN:
-    { switch((int) wParam)
+    { DEBUG(NAME_key, Cprintf("WM_KEYDOWN(%d)\n", (int) wParam));
+
+      switch((int) wParam)
       { case VK_DELETE:		id = toInt(127);	break;
         case VK_LEFT:		id = NAME_cursorLeft;	break;
         case VK_RIGHT:		id = NAME_cursorRight;	break;
@@ -320,15 +399,19 @@ messageToKeyId(UINT message, UINT wParam, LONG lParam, unsigned long *bmask)
 	  if ( state & BUTTON_control )
 	    id = ZERO;
 	  break;
-	case 0xbd:			/* OEM specific Control('_') ??? */
+	case VK_OEM_MINUS:
 	  if ( (state & BUTTON_control) && !(state & BUTTON_shift) )
-	    id = toInt(Control('_'));
+	    id = toInt('-');
 	  break;
-	case 0x56:			/* OEM specific 'V' ??? */
+	case VK_OEM_PLUS:
+	  if ( (state & BUTTON_control) )
+	    id = toInt('+');
+	  break;
+	case 'V':			/* OEM specific 'V' ??? */
 	  if ( (state & BUTTON_control) && (state & BUTTON_meta) )
 	    id = toInt(Control('V') + META_OFFSET);
 	  break;
-	case 0x49:			/* OEM specific 'I' ??? */
+	case 'I':			/* OEM specific 'I' ??? */
 	  if ( (state & BUTTON_control) && (state & BUTTON_meta) )
 	    id = toInt(Control('I') + META_OFFSET);
 	  break;
@@ -337,13 +420,15 @@ messageToKeyId(UINT message, UINT wParam, LONG lParam, unsigned long *bmask)
       break;
     }
     case WM_SYSCHAR:			/* handle ALT keys myself */
+      DEBUG(NAME_key, Cprintf("WM_SYSCHAR(%d)\n", (int) wParam));
       id = toInt(wParam + META_OFFSET);
       state |= BUTTON_meta;
       break;
 
     case WM_CHAR:
-    { id = toInt(wParam);
+    { DEBUG(NAME_key, Cprintf("WM_CHAR(%d)\n", (int) wParam));
 
+      id = toInt(wParam);
       if ( wParam == ' ' &&  (state & BUTTON_control) )
         id = ZERO;			/* ^-space --> ^@ */
       else if ( wParam == 8 && !(state & BUTTON_control) )
@@ -492,9 +577,12 @@ messageToEvent(HWND hwnd, UINT message, UINT wParam, LONG lParam)
     }
 #ifdef WM_MOUSEWHEEL			/* appears to be sent to frame! */
     case WM_MOUSEWHEEL:
-    { id = NAME_wheel;
+    { short a = (short)HIWORD(wParam);
+      Any angle = toInt(a);
+
+      id = NAME_wheel;
       ctx_name = NAME_rotation;
-      ctx = toInt(HIWORD(wParam));
+      ctx = angle;
 
       mouse_ev++;
       break;
@@ -569,7 +657,9 @@ messageToEvent(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		      EAV);
 
     if ( ctx_name )
+    { DEBUG(NAME_event, Cprintf("\tcontext %s = %s\n", pp(ctx_name), pp(ctx)));
       attributeObject(ev, ctx_name, ctx);
+    }
 
     return ev;
   } else
