@@ -1,11 +1,10 @@
-/*  $Id$
-
-    Part of XPCE --- The SWI-Prolog GUI toolkit
+/*  Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
-    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2002, University of Amsterdam
+    E-mail:        J.Wielemaker@vu.nl
+    WWW:           http://www.swi-prolog.org/projects/xpce/
+    Copyright (C): 1985-2012, University of Amsterdam
+			      VU University Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -44,9 +43,6 @@
 #endif
 #endif
 
-#undef offset
-#define offset(s, f) ((int)&((s *)NULL)->f)
-
 /* Using ws_ref for storing the bits and the xref mechanism for storing
    the Windows HBITMAP handle
 */
@@ -307,7 +303,9 @@ while the structure in memory is 32-bit aligned.  Hence, we read 3 times
 sizeof(WORD).
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#ifndef offsetof
 #define offsetof(type, field) (int)(&((type *)NULL)->field)
+#endif
 
 typedef struct
 { BYTE	bWidth;               /* Width of the image */
@@ -672,7 +670,7 @@ of that, and left the code for later.  This is the XPMTODIB.
     int n;
     int pentries=0;
 
-    lp = pceMalloc(offset(LOGPALETTE, palPalEntry[xpmimg->ncolors]));
+    lp = pceMalloc(offsetof(LOGPALETTE, palPalEntry[xpmimg->ncolors]));
     lp->palVersion    = 0x300;
     pe                = &lp->palPalEntry[0];
 
@@ -1929,24 +1927,24 @@ black_mask(int w, int h)
 
 
 HICON
-ws_icon_from_image(Image img)
-{ WsImage r = attach_ws_image(img);
+ws_icon_from_image(Image image)
+{ WsImage r = attach_ws_image(image);
 
   if ( !r->icon )
-  { int iw = GetSystemMetrics(SM_CXICON);
-    int ih = GetSystemMetrics(SM_CYICON);
-    int freemask = FALSE;
-    Image image, imask;
+  { int freemask = FALSE;
+    Image imask;
+    ICONINFO iinfo;
     HICON icon;
+    HBITMAP hbmColor;
 
-#ifdef O_SCALE_ICON
-    if ( valInt(img->size->w) != iw || valInt(img->size->h) != ih )
-      image = get(img, NAME_scale,
-		  answerObject(ClassSize, toInt(iw), toInt(ih), EAV),
-		  EAV);
-    else
-#endif
-      image = img;
+						/* first open the main image */
+						/* to initialize the mask */
+    if ( isNil(image->display) )
+      assign(image, display, CurrentDisplay(NIL));
+    hbmColor = (HBITMAP) getXrefObject(image, image->display);
+
+    DEBUG(NAME_icon, Cprintf("ws_icon_from_image(%s) (mask = %s)\n",
+			     pp(image), pp(image->mask)));
 
     if ( notNil(image->mask) )
     { if ( image->mask->kind == NAME_pixmap )
@@ -1954,52 +1952,19 @@ ws_icon_from_image(Image img)
 	freemask = TRUE;
       } else
 	imask = image->mask;
-    } else
-      imask = black_mask(iw, ih);
+    } else					/* Still no mask; create one */
+      imask = black_mask(valInt(image->size->w), valInt(image->size->h));
 
-    if ( isNil(image->display) )
-      assign(image, display, CurrentDisplay(NIL));
     assign(imask, display, image->display);
 
-#if 1
-  { ICONINFO iinfo;
-
     iinfo.fIcon = TRUE;
     iinfo.xHotspot = 0;
     iinfo.yHotspot = 0;
     iinfo.hbmMask  = (HBITMAP) getXrefObject(imask, imask->display);
-    iinfo.hbmColor = (HBITMAP) getXrefObject(image, image->display);
+    iinfo.hbmColor = hbmColor;
 
     icon = CreateIconIndirect(&iinfo);
-  }
-#else
-  { HBITMAP hbm, hmsk;
-    BITMAP bm, msk;
-    BYTE *bmbits, *mskbits;
 
-    hbm  = (HBITMAP) getXrefObject(image, image->display);
-    hmsk = (HBITMAP) getXrefObject(imask, imask->display);
-    GetObject(hbm,  sizeof(bm), &bm);
-    GetObject(hmsk, sizeof(msk), &msk);
-    bmbits  = pceMalloc(ih * bm.bmWidthBytes);
-    mskbits = pceMalloc(ih * msk.bmWidthBytes);
-
-    GetBitmapBits(hbm,  ih * bm.bmWidthBytes,  bmbits);
-    GetBitmapBits(hmsk, ih * msk.bmWidthBytes, mskbits);
-
-    icon = CreateIcon(PceHInstance,
-		      iw, ih,
-		      (BYTE)bm.bmPlanes, (BYTE)bm.bmBitsPixel,
-		      mskbits,
-		      bmbits);
-
-    pceFree(bmbits);
-    pceFree(mskbits);
-  }
-#endif
-
-    if ( image != img )
-      freeObject(image);
     if ( freemask )
       freeObject(imask);
 
